*Version 2.0
 Written by Carrie Walker;

***************************************************************************************
Instructions:
Download the 2023 Dialysis Event MRAT data from REDCap by using the 'Data Exports, Reports, and Stats' option
on the left-hand menu. Be sure to export data from the Dialysis Event instument only, and as a .csv 
format using variable names, not labels. Save the file in a secure location and enter the filepath below.

This code works with 2023 Dialysis Event REDCap data only.

For step by step instructions on exporting data, follow steps 1-5 of the How-To document located
at the link below. You DO NOT need to deidentify your data for this code and report. Please ignore
that step in the guidance document.
	https://www.cdc.gov/nhsn/pdfs/validation/2022/export-data-from-REDCap.pdf

Enter the facility's name and orgID below, where prompted. Do not use quotes, and be sure the semicolons 
aren't deleted.

Do not edit anywhere else in the code after these macros.
***************************************************************************************;

*Enter the filepath, including the file name and extension;
%let path = C:\Filepath\DE MRAT data.csv;

*Enter the facility's name, exactly as you want it written in the report.;
%let facname = Dialysis Facility Name;

*Enter facility's orgID;
%let orgid= 1234;

proc import datafile="&path" 
out=rcde (where=(orgid_de_2023 =&orgid))
dbms=csv replace;
guessingrows=2000;
run;

proc format;
value ivammc
1="I) Misinterpretation of antibiotic administration or reporting criteria"
2="II) Missing data elements or incorrect information reported"
3="III) Other, specify";
value $ ivammcI 
"a"="a) A one-time or prophylactic dose was given"
"b"="b) Related blood culture results were negative"
"c"="c) IVAMS is for non-vascular access problem (e.g., pneumonia)"
"d"="d) First outpatient IVAM administration resulting from continuation of inpatient treatment" 
"e"="e) Administered on outpatient basis, outside of dialysis facility (e.g., LTC, prison)" 
"f"="f) Facility thought they were only supposed to report specific microbials (e.g., vanco)" 
"g"="g) Misapplication of 21-day rule; event was within 21 days of previous IVAM event"
"h"="h) Reported for an oral antibiotic" 
"i"="i) Not reported when drug not owned by clinic"
"j"="j) Not reported for deceased patients or patients transferred out of facility";
value $ ivammcII
"a"= "a) Did not report all event elements (e.g., reported PBC but not IVAM)" 
"b"= "b) Event Date incorrect" 
"c"= "c) Vascular Access type incorrect" 
"d"= "d) Other, specify";

value pbcmc
1="I) Misinterpretation of PBC reporting criteria"
2="II) Missing data elements or incorrect information reported"
3="III) Other, specify";
value $ pbcmcI 
"a"= "a) Source of infection reported as contamination or uncertain" 
"b"= "b) IV antimicrobials were not started" 
"c"= "c) Attributed to non-vascular access problems (e.g., wound, pneumonia)" 
"d"= "d) Obtained within first day after hospital admit" 
"e"= "e) Misapplication of 21-day rule; event was within 21 days of previous PBC event" 
"f"= "f) Not reported for deceased patients or patients transferred out of facility";
value $ pbcmcII 
"a"= "a) Did not report all event elements (e.g., reported IVAM but not PBC)" 
"b"= "b) Event Date incorrect" 
"c"= "c) Vascular Access type incorrect" 
"d"= "d) Other, specify";

value prsmc
1="I) Misinterpretation of PRS identification or reporting criteria"
2="II) Missing data elements or incorrect information reported"
3="III) Other, specify";
value $ prsmcI 
"a"="a) Not reported because antimicrobials were not started" 
"b"="b) Infection suspected but not confirmed and/or doctor says PRS is cellulitis" 
"c"="c) Facility lacks process to track PRS events" 
"d"="d) Misapplication of 21-day rule; event was within 21 days of previous PRS event" 
"e"="e) Not reported for deceased patients or patients transferred out of facility";
value $ prsmcII 
"a"="a) Did not report all event elements (e.g., reported IVAM but not PRS)" 
"b"="b) Event Date incorrect" 
"c"="c) Vascular Access type incorrect" 
"d"="d) Other, specify";
value resultstat
1="True Positive"
2="False Negative"
3="False Positive"
4="True Negative" 
9="Review";
value yesno
1="Yes"
0="No";
run;

data outcome_de;
set rcde ;
keep orgid_de: record_id patientid_de: ivam: pbc: prs: num:;
run;

proc sort data=outcome_de; by orgid_de_2023 record_id patientid_de_2023; run;
proc transpose data=outcome_de out=trans_de;
by orgid_de_2023 record_id patientid_de_2023;
var ivam: pbc: prs: ;
run;
proc sort data=trans_de; by record_id _name_; run;

data de2;
set trans_de;
length Drug $50. pathogen _mcreas2oth _mcreas3 $100.;
rename orgid_de_2023=orgid_de patientid_de_2023=patientid_de;
col2=strip(col1);
if col2 in ("." "") then delete;
if index(_name_,"prob_de") then delete;
if (index(_name_, "varep") > 0 or index(_name_, "vapres") > 0 or index(_name_, "accaff") > 0 or index(_name_, "sx_de") > 0) and col2="0" then delete;

if index(_name_,"drug_de") > 0 then Drug=col2;
if index(_name_,"startdt") > 0 then startdt=input(col2,mmddyy10.);
if index(_name_, "path") > 0 then pathogen=col2;
if index(_name_,"colldt") > 0 then Colldt=input(col2,mmddyy10.);
if index(_name_,"onsetdt") > 0 then onsetdt=input(col2,mmddyy10.);

if index(_name_,"reportable") > 0 then reportable=input(col2,1.);
if index(_name_,"reported") > 0 then reported=input(col2,1.);
if index(_name_,"repeventdt") > 0 then repdt=input(col2,mmddyy10.);

if index(_name_, "vapres_de_2023___f") > 0 and col2="1" then _VApresF="Fistula";
if index(_name_, "vapres_de_2023___g") > 0 and col2="1" then _VApresG="Graft";
if index(_name_, "vapres_de_2023___t") > 0 and col2="1" then _VApresT="Tunneled CL";
if index(_name_, "vapres_de_2023___nt") > 0 and col2="1" then _VApresNT="Non-Tunneled CL";
if index(_name_, "vapres_de_2023___o") > 0 and col2="1" then _VApresO="Other";
if index(_name_, "vapres_de_2023___na") > 0 and col2="1" then _VApresNA="None";

if index(_name_, "varep_de_2023___f") > 0 and col2="1" then _VArepF="Fistula";
if index(_name_, "varep_de_2023___g") > 0 and col2="1" then _VArepG="Graft";
if index(_name_, "varep_de_2023___t") > 0 and col2="1" then _VArepT="Tunneled CL";
if index(_name_, "varep_de_2023___nt") > 0 and col2="1" then _VArepNT="Non-Tunneled CL";
if index(_name_, "varep_de_2023___o") > 0 and col2="1" then _VArepO="Other";
if index(_name_, "varep_de_2023___na") > 0 and col2="1" then _VArepNA="None";

if index(_name_, "accaff_de_2023___f") > 0 and col2="1" then _VAaffF="Fistula";
if index(_name_, "accaff_de_2023___g") > 0 and col2="1" then _VAaffG="Graft";
if index(_name_, "accaff_de_2023___t") > 0 and col2="1" then _VAaffT="Tunneled CL";
if index(_name_, "accaff_de_2023___nt") > 0 and col2="1" then _VAaffNT="Non-Tunneled CL";
if index(_name_, "accaff_de_2023___o") > 0 and col2="1" then _VAaffO="Other";

if index(_name_, "sx_de_2023___p") > 0 and col2="1" then sx_p="Pus";
if index(_name_, "sx_de_2023___r") > 0 and col2="1" then sx_r="Redness";
if index(_name_, "sx_de_2023___s") > 0 and col2="1" then sx_s="Swelling";

if substr(_name_,1,4) =  "ivam" then do;
if index(_name_, "misclassreason") > 0 and col2 in ("1" "2" "3") then _mcreason=put(input(col2,1.), ivammc.);
if index(_name_, "misreason1") > 0 and col2 ne "" then _mcreas1=put(col2, ivammcI.);
if index(_name_, "misreason2") > 0 and col2 ne "" then _mcreas2=put(col2, ivammcII.);
if index(_name_, "misreason2_oth") > 0 and col2 ne "" then _mcreas2oth=col2;
if index(_name_, "misreason3") > 0 and col2 ne "" then _mcreas3=col2;
if index(_name_, "ivam10") > 0 then _ivamnum=10;
else _ivamnum=input(substr(_name_,5,1), 2.);end;

if substr(_name_,1,3) =  "pbc" then do;
if index(_name_, "misclassreason") > 0 and col2 in ("1" "2" "3") then _mcreason=put(input(col2,1.), pbcmc.);
if index(_name_, "misreason1") > 0 and col2 ne "" then _mcreas1=put(col2, pbcmcI.);
if index(_name_, "misreason2") > 0 and col2 ne "" then _mcreas2=put(col2, pbcmcII.);
if index(_name_, "misreason2_oth") > 0 and col2 ne "" then _mcreas2oth=col2;
if index(_name_, "misreason3") > 0 and col2 ne "" then _mcreas3=col2;
if index(_name_, "pbc10") > 0 then _pbcnum=10;
else _pbcnum=input(substr(_name_,4,1), 2.); end;

if substr(_name_,1,3) =  "prs" then do;
if index(_name_, "misclassreason") > 0 and col2 in ("1" "2" "3") then _mcreason=put(input(col2,1.), prsmc.);
if index(_name_, "misreason1") > 0 and col2 ne "" then _mcreas1=put(col2, prsmcI.);
if index(_name_, "misreason2") > 0 and col2 ne "" then _mcreas2=put(col2, prsmcII.);
if index(_name_, "misreason2_oth") > 0 and col2 ne "" then _mcreas2oth=col2;
if index(_name_, "misreason3") > 0 and col2 ne "" then _mcreas3=col2; 
if index(_name_, "prs10") > 0 then _prsnum=10;
else _prsnum=input(substr(_name_,4,1), 2.); end;

format startdt colldt onsetdt repdt mmddyy10.;
run;

proc sql; create table de3 as select orgid_de, patientid_de, _ivamnum, 
_pbcnum, _prsnum, 
max(Drug) as Drug, max(pathogen) as pathogen, max(colldt) as colldt format mmddyy10.,
max(startdt) as startdt format mmddyy10., max(onsetdt) as onsetdt format mmddyy10.,
max(reportable) as reportable, max(reported) as reported, max(repdt) as repdt format mmddyy10., 
max(_vapresf) as _vapresf, max(_vapresg) as _vapresg, max(_vaprest) as _vaprest, max(_vapresnt) as _vapresnt,
max(_vapreso) as _vapreso, max(_vapresna) as _vapresna, 
max(_varepf) as _varepf, max(_varepg) as _varepg, max(_varept) as _varept, max(_varepnt) as _varepnt,
max(_varepo) as _varepo, max(_varepna) as _varepna, 
max(_vaafff) as _vaafff, max(_vaaffg) as _vaaffg, max(_vaafft) as _vaafft, max(_vaaffnt) as _vaaffnt,
max(_vaaffo) as _vaaffo, max(sx_p) as sx_p, max(sx_r) as sx_r, max(sx_s) as sx_s,
max(_mcreason) as _mcreason, max(_mcreas1) as _mcreas1, max(_mcreas2) as _mcreas2, 
max(_mcreas2oth) as _mcreas2oth, max(_mcreas3) as _mcreas3
from de2
group by orgid_de, patientid_de, _ivamnum, _pbcnum, _prsnum;
quit;

data de4 needsreview;
set de3 (rename= ( _ivamnum = IVAMnumber _pbcnum = PBCnumber _prsnum = PRSnumber));
length VAaffected VApresent VAreported SXreported $50. pathogen $100. misclassreason $200.;
VAaffected=catx(", ", of _vaaff:);
	if VAaffected = "" then VAaffected="n/a";
VApresent=catx(", ", of _vapres:);
	if VApresent = "" then VApresent="n/a";
VAreported=catx(", ", of _varep:);
	if VAreported = "" then VAreported="n/a";

SXreported=catx(", ", of sx_:);
misclassreason=catx(": ", of _mcreas:);

if ivamnumber ne . then EventType="IVAM";
if pbcnumber ne . then EventType="PBC";
if prsnumber ne . then EventType="PRS";

if drug = "" and startdt = . and reportable = . and reported = . then delete;
if pathogen = "" and colldt = . and reportable = . and reported = . then delete;
if onsetdt = . and reportable = . and reported = . then delete;

*Creating tp/fp/tn/fn;
if reportable=1 and reported=1 then do;
	Validator=1; Facility=1; Resultstatus=1; end; *TP;
if reportable=0 and reported=0 then do;
	Validator=0; Facility=0; Resultstatus=4; end; *TN;
if reportable=1 and reported=0 then do;
	Validator=1; Facility=0; Resultstatus=2; end; *FN;
if reportable=0 and reported=1 then do;
	Validator=0; Facility=1; Resultstatus=3; end; *FP;
if reportable in (1 0) and reported=. then Resultstatus=9; *review;
if reportable=. and reported in (1 0) then Resultstatus=9; *review;

label orgid_de= "OrgID" patientid_de="Patient ID" reportable='Reportable to NHSN' 
pathogen="Pathogen(s) Identified" colldt="Specimen Collection Date" onsetdt="Symptom Onset Date" 
reported='Reported by Facility' drug="IVAM Drug Name" startdt="IVAM Start Date" 
repdt="Report Date" vapresent="VA Present per chart" vareported="VA Reported to NHSN" VAaffected="VA Affected" 
misclassreason="Misclassification Reason(s)" SXreported="Reported Symptoms";
format reportable reported yesno.;
drop _:; label;

if resultstatus in (1:4) then output de4;
if resultstatus=9 then output needsreview;
run;

proc sort data=de4; by EventType descending Facility descending Validator; run;
proc freq data=de4 order=data;
by orgid_de EventType; 
table facility*validator/nocol nocum norow nopercent out=de_freqsevtype;
format facility Validator yesno.;
run;
proc freq data=de4 order=data;
by orgid_de; 
table facility*validator/nocol nocum norow nopercent out=de_freqstotal;
format facility Validator yesno.;
run;

data de_freqs;length eventtype fac $5.;
set de_freqsevtype de_freqstotal;
if eventtype="" then eventtype="Total";
percent2=round(percent, 1.);
fac=put(facility, yesno.);
drop percent; rename percent2 =percent;
run;

proc sql; create table de_peragreement as select orgid_de, eventtype,
sum((case when facility=1 and validator=1 then count else 0 end) + 
		(case when facility=0 and validator=0 then count else 0 end)) as numerator,
sum(count) as denom, put(round(calculated numerator/(calculated denom)*100,.1),4.1) as PerAgree
from de_freqs 
group by orgid_de, eventtype;
quit;

data _null_;
set de_peragreement;
if eventtype = "IVAM" then call symput ('ivampa', peragree);
if eventtype = "PBC" then call symput ('pbcpa', peragree);
if eventtype = "PRS" then call symput ('prspa', peragree);
if eventtype = "Total" then call symput ('overallpa', peragree);
call scan("&path", -1, pos, length,"\");
call symput("reppath", substr("&path",1,pos-2) || "\Dialysis - &facname Validation Summary Report %sysfunc(date(),mmddyyd10.).xlsx");
run; 
%put &ivampa &pbcpa &prspa &overallpa;
%put &reppath;

proc sort data=de4; by orgid_de eventtype patientid_de colldt startdt onsetdt; run;

*Creating report;
ods excel file = "&reppath" 
options(sheet_interval='none' embedded_titles='yes' embedded_footnotes='yes'
sheet_name = "2x2 Tables");options missing=0;
title2;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype = "Total";
title1 c=black h=11pt j=center "&facname Dialysis Event Validation Summary";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "Overall Percent Agreement = %SYSFUNC(strip(&overallpa))%"; endcomp;
compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 1. Overall Dialysis Event Validation Agreement"; endcomp;
run;
title1;
title2;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype="IVAM";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "IVAM Percent Agreement = %SYSFUNC(strip(&ivampa))%"; endcomp;
	compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 2. IVAM Event Validation"; endcomp;
run;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype="PBC";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "PBC Percent Agreement = %SYSFUNC(strip(&pbcpa))%"; endcomp;
	compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 3. PBC Event Validation"; endcomp;
run;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype="PRS";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "PRS Percent Agreement = %SYSFUNC(strip(&prspa))%"; endcomp;
	compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 4. PRS Event Validation"; endcomp;
run;
  footnote;

*Linelists;
ods excel options(sheet_name = "IVAM Linelist" sheet_interval="now" flow="header, data");options missing="";
proc report data=de4 nowd 
style(column)=[just=center frame=box]
style(header)=[background=lightblue font_size=8pt color=black  bordercolor=black cellspacing=1];label;
where eventtype="IVAM";
column orgid_de patientid_de Drug startdt reportable reported repdt VApresent VAreported misclassreason;
define misclassreason / style(column)=[width=350pt tagattr='wrap:yes' just=left];
run;

ods excel options(sheet_name = "PBC Linelist" sheet_interval="now" flow="header, data");
proc report data=de4 nowd 
style(column)=[just=center frame=box]
style(header)=[background=lightblue font_size=8pt color=black  bordercolor=black cellspacing=1];label;
where eventtype="PBC";
column orgid_de patientid_de colldt pathogen reportable reported repdt VApresent VAreported misclassreason;
define misclassreason / style(column)=[tagattr='wrap:yes' just=left];
run;

ods excel options(sheet_name = "PRS Linelist" sheet_interval="now" flow="header, data");
proc report data=de4 nowd 
style(column)=[just=center frame=box]
style(header)=[background=lightblue font_size=8pt color=black  bordercolor=black cellspacing=1];label;
where eventtype="PRS";
column orgid_de patientid_de onsetdt VAaffected SXreported reportable reported repdt VApresent VAreported misclassreason ;
define misclassreason / style(column)=[tagattr='wrap:yes' just=left];
	 run;
ods excel close;
options missing=.;